import {
  Component,
  OnInit,
  ChangeDetectorRef,
  OnDestroy,
  Output,
  EventEmitter
} from '@angular/core';
import { SelectedDept, UserInfoSS } from '@ucap-webmessenger/protocol-query';
import { NGXLogger } from 'ngx-logger';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from '@ucap-webmessenger/ui';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { Store, select } from '@ngrx/store';

import * as AppStore from '@app/store';
import * as QueryStore from '@app/store/messenger/query';
import * as SyncStore from '@app/store/messenger/sync';
import * as ChatStore from '@app/store/messenger/chat';
import * as StatusStore from '@app/store/messenger/status';
import { Observable, Subscription, combineLatest } from 'rxjs';
import { PresenceType, StatusCode } from '@ucap-webmessenger/core';
import {
  StatusBulkInfo,
  WorkStatusType
} from '@ucap-webmessenger/protocol-status';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { KEY_VER_INFO } from '@app/types';
import { Sort } from '@angular/material/sort';

@Component({
  selector: 'app-layout-messenger-organization',
  templateUrl: './organization.component.html',
  styleUrls: ['./organization.component.scss']
})
export class OrganizationComponent implements OnInit, OnDestroy {
  @Output()
  openProfile = new EventEmitter<number>();

  sessionVerinfo: VersionInfo2Response;

  isSearch: boolean;
  isSearchSubscription: Subscription;
  selectedDepartmentProcessing$: Observable<boolean>;
  selectedDepartment$: Observable<SelectedDept>;

  departmentUserInfoListSubscription: Subscription;
  searchDepartmentUserInfoListSubscription: Subscription;

  selectedDepartmentUserInfoList: UserInfoSS[] = [];

  profileImageRoot: string;
  presence$: Observable<StatusBulkInfo[]>;
  PresenceType = PresenceType;

  constructor(
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private changeDetectorRef: ChangeDetectorRef,
    private logger: NGXLogger
  ) {
    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
  }

  ngOnInit() {
    this.isSearchSubscription = this.store
      .pipe(select(AppStore.MessengerSelector.QuerySelector.isSearch))
      .subscribe(isSearch => {
        this.isSearch = isSearch;
      });

    this.selectedDepartmentProcessing$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.selectedDepartmentProcessing
      )
    );

    this.selectedDepartment$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.selectedDepartment)
    );

    this.departmentUserInfoListSubscription = this.store
      .pipe(
        select(AppStore.MessengerSelector.QuerySelector.departmentUserInfoList)
      )
      .subscribe(list => {
        if (!this.isSearch) {
          this.selectedDepartmentUserInfoList = list;
        }
      });

    this.searchDepartmentUserInfoListSubscription = this.store
      .pipe(
        select(
          AppStore.MessengerSelector.QuerySelector.searchDepartmentUserInfoList
        )
      )
      .subscribe(list => {
        if (!!this.isSearch) {
          this.selectedDepartmentUserInfoList = list;
        }
      });

    this.presence$ = this.store.pipe(
      select(AppStore.MessengerSelector.StatusSelector.selectAllStatusBulkInfo)
    );

    this.profileImageRoot = this.sessionVerinfo.profileRoot;
  }

  ngOnDestroy(): void {
    if (!!this.isSearchSubscription) {
      this.isSearchSubscription.unsubscribe();
    }
    if (!!this.departmentUserInfoListSubscription) {
      this.departmentUserInfoListSubscription.unsubscribe();
    }
    if (!!this.searchDepartmentUserInfoListSubscription) {
      this.searchDepartmentUserInfoListSubscription.unsubscribe();
    }
  }

  onClickOpenProfile(userSeq: number) {
    this.openProfile.emit(userSeq);
  }
}
